
/*
	"EventLogger" SoundPlay general purpose plugin.
	Copyright 2000 Marco Nelissen (marcone@xs4all.nl)

	Permission is hereby granted to use this code for creating other
	SoundPlay-plugins.
*/

#include <Application.h>
#include <MessageRunner.h>
#include <Roster.h>
#include <stdio.h>
#include "EventLogger.h"

// some liblayout headers.
// make sure you have the liblayout development kit
// installed, and set the paths in the project settings
// of this project.
#include "MBorder.h"
#include "MScrollView.h"

void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *, SoundPlayBufferedReader *reader);
void destroyplugin(void*,void *_loader);
status_t show(void*);
void hide(void*);
void setconfig(void*,BMessage *config);
void getconfig(void*,BMessage *config);

const char *PLUGINID="sp-logger";

static plugin_descriptor plugin={
	PLUGIN_DESCRIPTOR_MAGIC,PLUGIN_DESCRIPTOR_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,

	"Event logger",
	"By Marco Nelissen.\n",
	NULL,
	NULL, // configure
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_INTERFACE_MAGIC, PLUGIN_INTERFACE_VERSION,
	show,
	hide,
	setconfig,
	getconfig
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}



void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info, SoundPlayBufferedReader *reader)
{
	LoggerWindow *win=new LoggerWindow(info->controller);
	info->controller->AddWindow(win,FILTER_HOTKEYS); // let SoundPlay handle some hotkeys for this window
	*data=win;
	return &plugin_ops;
}

void destroyplugin(void *,void *data)
{
	LoggerWindow *win=(LoggerWindow*)data;
	win->Lock();
	win->Quit();
}

status_t show(void *data)
{
	LoggerWindow *win=(LoggerWindow*)data;
	win->Show();
	return B_OK;
}

void hide(void *data)
{
	LoggerWindow *win=(LoggerWindow*)data;
	win->Hide();
}

void setconfig(void *data,BMessage *config)
{
	LoggerWindow *win=(LoggerWindow*)data;
	BRect rect;
	if(B_OK==config->FindRect("frame",&rect))
	{
		win->MoveTo(rect.LeftTop());
		win->ResizeTo(rect.Width(),rect.Height());
	}
}
void getconfig(void *data,BMessage *config)
{
	LoggerWindow *win=(LoggerWindow*)data;
	config->MakeEmpty();
	config->AddRect("frame",win->Frame());
}



LoggerWindow::LoggerWindow(SoundPlayController *controller)
	: MWindow(BRect(100,100,300,350),"Eventlogger",B_TITLED_WINDOW,B_ASYNCHRONOUS_CONTROLS)
{
	ctrl=controller;
	AddChild
	(
		new MBorder
		(
			M_RAISED_BORDER,8,"",
			new MScrollView
			(
				lv=new MListView(),
				false,
				true			
			)
		)
	);
	runner=new BMessageRunner(BMessenger(this),new BMessage('ping'),500000);
}

LoggerWindow::~LoggerWindow()
{
	delete runner;
}

void LoggerWindow::Show()
{
	// This UI will only use 2 tracks. Delete the rest.
	ctrl->Lock();
	LogMessage("Starting up");
	ctrl->AddListener(this);
	for(int i=0;;i++)
	{
		PlaylistPtr playlist=ctrl->PlaylistAt(i);
		if(!playlist->Lock())
			break;
		LogMessage("Found track");
		playlist->AddListener(this);
		playlist->Unlock();
	}
	LogMessage("Listening");
	ctrl->Unlock();
	MWindow::Show();
}

void LoggerWindow::LogMessage(const char *message,int32 parameter)
{
	char string[1000];
	sprintf(string,message,parameter);
	lv->AddItem(new BStringItem(string));
	lv->ScrollTo(lv->ItemFrame(lv->CountItems()-1).LeftTop());
}

void LoggerWindow::MessageReceived(BMessage *mes)
{
	// re-add the playlists to the window in case they got orphaned
	switch(mes->what)
	{
		case PLAYLIST_ADD:
			{
				ulong	type;
				long	count;
				mes->GetInfo("ID", &type, &count);
				if(count==1)
					LogMessage("playlist: 1 entry added");
				else
					LogMessage("playlist: %d entries added",count);
			}
			break;
		case PLAYLIST_REMOVE:
			LogMessage("playlist: entry removed");
			break;
		case PLAYLIST_EMPTY:
			LogMessage("playlist: cleared");
			break;
		case PLAYLIST_DESTROYED:
			LogMessage("playlist: destroyed");
			break;
		case PLAYLIST_LOST_CONTROLS:
			LogMessage("playlist: lost controls");
			break;
		case PLAYLIST_GAINED_CONTROLS:
			LogMessage("playlist: gained controls");
			break;
		case PLAYLIST_CURRENT:
			{
				int32 newindex=mes->FindInt32("ID");
				LogMessage("playlist: current index changed to ID %d",newindex);
			}
			break;
		case CONTROLLER_ADD:
			{
				int32 id;
				if(B_OK==mes->FindInt32("Playlist",&id))
				{
					ctrl->Lock();
					PlaylistPtr playlist(id);
					if(playlist->Lock())
					{
						playlist->AddListener(this);
						playlist->Unlock();
						LogMessage("controller: track added");
					}
					else
						LogMessage("controller: invalid track added");
					ctrl->Unlock();
				}
				else
					LogMessage("ID-less track added");
			}			
			break;
		case CONTROLLER_REMOVE:
			LogMessage("controller: track removed");
			break;
			
		case 'ping':
			{
				ctrl->Lock();
				char title[1000];
				int32 numitems=ctrl->CountPlaylists();
				sprintf(title,"Eventlogger: %d track%s",numitems,numitems>1?"s":"");
				ctrl->Unlock();
				SetTitle(title);
			}
			break;

		default:
			MWindow::MessageReceived(mes);
			break;
	}
}

bool LoggerWindow::QuitRequested()
{
	ctrl->DisableInterface(PLUGINID);
	return false;
}

int main()
{
	BApplication app("application/x-vnd.marcone-spmixer");
	
	BMessage msg(ENABLE_INTERFACE);
	msg.AddString("interface",PLUGINID);
	be_roster->Launch("application/x-vnd.marcone-soundplay",&msg);
	return 0;
}

